import pygame, cPickle, os
from OpenGL.GL import *

def MTL(filename):
    contents = {}
    mtl = None
    for line in open(os.path.join("dev-data", filename), "r"):
        if line.startswith('#'): continue
        values = line.split()
        if not values: continue
        if values[0] == 'newmtl':
            mtl = contents[values[1]] = {}
        elif mtl is None:
            raise ValueError, "mtl file doesn't start with newmtl stmt"
        elif values[0] == 'map_Kd':
            # load the texture referred to by this declaration
            mtl[values[0]] = values[1]
        else:
            mtl[values[0]] = map(float, values[1:])
    return contents
 
class OBJ:
    def __init__(self, filename, swapyz=True):
        import ctypes

        self.vertices = []
        self.normals = []
#        self.texcoords = []
#        self.faces = []
        self.indices = {}

        vs, ns = [], []
        imap = {}

        material = None
        for line in open(os.path.join("dev-data", "%s.obj" % filename), "r"):
            if line.startswith('#'): continue
            values = line.split()
            if not values: continue
            if values[0] == 'v':
                v = map(float, values[1:4])
                if swapyz:
                    v = v[0], v[2], v[1]
                vs.append(v)
            elif values[0] == 'vn':
                v = map(float, values[1:4])
                if swapyz:
                    v = v[0], v[2], v[1]
                ns.append(v)
            elif values[0] == 'vt':
                self.texcoords.append(map(float, values[1:3]))
            elif values[0] in ('usemtl', 'usemat'):
                material = values[1]
            elif values[0] == 'mtllib':
                self.mtl = MTL(values[1])
                for material in self.mtl:
                    self.indices[material] = []
            elif values[0] == 'f':
#                face = []
#                texcoords = []
#                norms = []
                if len(values) < 4: continue
                for v in values[1:]:
                    w = v.split('/')
                    f, n = int(w[0]), int(w[2])
                    if (f,n) not in imap:
                        imap[(f,n)] = len(self.vertices)
                        self.vertices.append(vs[f-1])
                        self.normals.append(ns[n-1])
                    self.indices[material].append(imap[(f,n)])
                    
#                    face.append(int(w[0]))
#                    if len(w) >= 2 and len(w[1]) > 0:
#                        texcoords.append(int(w[1]))
#                    else:
#                        texcoords.append(0)
#                    if len(w) >= 3 and len(w[2]) > 0:
#                        norms.append(int(w[2]))
#                    else:
#                        norms.append(0)
#                self.faces.append((face, norms, texcoords, material))

        v = sum(self.vertices, ())
        a = (ctypes.c_float * len(v))(*v)
        self.vertices = ctypes.string_at(a, ctypes.sizeof(a))
        n = sum(self.normals, ())
        a = (ctypes.c_float * len(n))(*n)
        self.normals = ctypes.string_at(a, ctypes.sizeof(a))
        self.lens = {}
        for material, i in self.indices.items():
            a = (ctypes.c_int * len(i))(*i)
            self.indices[material] = ctypes.string_at(a, ctypes.sizeof(a))
            self.lens[material] = len(i)

    def generate(self, wireframe = False):

        self.gl_list = glGenLists(1)
        glNewList(self.gl_list, GL_COMPILE)
        glFrontFace(GL_CW)

        glVertexPointer(3, GL_FLOAT, 0, self.vertices)
        glNormalPointer(GL_FLOAT, 0, self.normals)
        
        if wireframe:
            glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
        else:
            glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)

        for material, mtl in self.mtl.items():
            if 'map_Kd' in mtl:
                surf = pygame.image.load(os.path.join("dev-data", mtl['map_Kd']))
                image = pygame.image.tostring(surf, 'RGBA', 1)
                ix, iy = surf.get_rect().size
                texid = mtl['texture_Kd'] = glGenTextures(1)
                glBindTexture(GL_TEXTURE_2D, texid)
                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
                glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, ix, iy, 0, GL_RGBA, GL_UNSIGNED_BYTE, image)

            glColor3fv(mtl["Kd"])
            glDrawElements(GL_TRIANGLES, self.lens[material], GL_UNSIGNED_INT, self.indices[material])

        glEndList()

    def render(self):
        glCallList(self.gl_list)

class Model:
    def __init__(self, name):
        pname = os.path.join("data", "%s.pickle" % name)
        if os.path.exists(pname):
            self.obj = cPickle.load(open(pname, "rb"))
        else:
            self.obj = OBJ(name)
            cPickle.dump(self.obj, open(pname, "wb"), 1)
        self.obj.generate()
    def render(self):
        self.obj.render()



if __name__ == "__main__":
    from pygame.locals import *
    import sys
    from OpenGL.GLU import *
    pygame.init()
    viewport = (800,600)
    hx = viewport[0]/2
    hy = viewport[1]/2
    srf = pygame.display.set_mode(viewport, OPENGL | DOUBLEBUF)
     
    glLightfv(GL_LIGHT0, GL_POSITION,  (-40, 200, 100, 0.0))
    glLightfv(GL_LIGHT0, GL_AMBIENT, (0.2, 0.2, 0.2, 1.0))
    glLightfv(GL_LIGHT0, GL_DIFFUSE, (0.5, 0.5, 0.5, 1.0))
    glEnable(GL_LIGHT0)
    glEnable(GL_LIGHTING)
    glEnableClientState(GL_VERTEX_ARRAY)
    glEnableClientState(GL_NORMAL_ARRAY)
    glEnable(GL_COLOR_MATERIAL)
    glEnable(GL_DEPTH_TEST)
    glShadeModel(GL_SMOOTH)           # most obj files expect to be smooth-shaded
    glEnable(GL_BLEND)
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
    glEnable(GL_TEXTURE_2D)


    # LOAD OBJECT AFTER PYGAME INIT
    obj = OBJ(sys.argv[1], swapyz=True)
    obj.generate()
     
    clock = pygame.time.Clock()
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    width, height = viewport
    gluPerspective(30.0, width/float(height), 0.1, 1000.0)
    glEnable(GL_DEPTH_TEST)
    glMatrixMode(GL_MODELVIEW)
     
    rx, ry = (0,0)
    tx, ty = (0,0)
    zpos = 5
    rotate = move = False
    while 1:
        clock.tick(30)
        for e in pygame.event.get():
            if e.type == QUIT:
                sys.exit()
            elif e.type == KEYDOWN and e.key == K_ESCAPE:
                sys.exit()
            elif e.type == MOUSEBUTTONDOWN:
                if e.button == 4: zpos = max(1, zpos-1)
                elif e.button == 5: zpos += 1
                elif e.button == 1: rotate = True
                elif e.button == 3: move = True
            elif e.type == MOUSEBUTTONUP:
                if e.button == 1: rotate = False
                elif e.button == 3: move = False
            elif e.type == MOUSEMOTION:
                i, j = e.rel
                if rotate:
                    rx += i
                    ry += j
                if move:
                    tx += i
                    ty -= j
     
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glLoadIdentity()
     
        # RENDER OBJECT
        glTranslate(tx/20., ty/20., - zpos)
        glRotate(ry, 1, 0, 0)
        glRotate(rx, 0, 1, 0)
        obj.render()
     
        pygame.display.flip()


